(***********************************************************************)
(*                                                                     *)
(*                           Objective Caml                            *)
(*                                                                     *)
(*            Jun Furuse, projet Cristal, INRIA Rocquencourt           *)
(*                                                                     *)
(*  Copyright 1999,2000                                                *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)
open Image
open OImage
open Ximage
open Gdk

let of_image visual progress img =
  let quick_color_create = Truecolor.color_creator visual in
  let quick_color_parser = Truecolor.color_parser visual in
  let prog v (* 0.0 .. 1.0 *) = 
    match progress with
    | Some f -> f v
    | None -> ()
  in
  match OImage.tag img with
  | Rgb24 bmp ->
      let width = bmp#width in
      let height = bmp#height in
      let ximg = create ~kind: `FASTEST ~visual ~width ~height in

      let f_height = float height in
      for y = 0 to height - 1 do
      	for x = 0 to width - 1 do 
    	  let rgb = bmp#unsafe_get x y in
    	  let red = rgb.r lsl 8 in
    	  let green = rgb.g lsl 8 in
    	  let blue = rgb.b lsl 8 in
    	  Image.put_pixel ximg.ximage ~x ~y 
	    ~pixel:(quick_color_create ~red ~green ~blue)
	done;
	prog  (float (y+1) /. f_height)
      done;
      ximg

  | Index8 bmp ->
      let width = bmp#width in
      let height = bmp#height in
      let cmap = bmp#colormap.map in
      let ximg = create ~kind: `FASTEST ~visual ~width ~height in

      let f_height = float height in
      for y = 0 to height - 1 do
      	for x = 0 to width - 1 do 
	  let rgb = 
	    try
	      cmap.(bmp#unsafe_get x y) 
	    with
	      _ -> {r=255; g=0; b=0} 
	  in
    	  let red = rgb.r lsl 8 in
    	  let green = rgb.g lsl 8 in
    	  let blue = rgb.b lsl 8 in
    	  Image.put_pixel ximg.ximage ~x ~y 
	    ~pixel:(quick_color_create ~red ~green ~blue)
	done;
	prog (float (y+1) /. f_height)
      done;
      ximg

  | Index16 bmp ->
      let width = bmp#width in
      let height = bmp#height in
      let cmap = bmp#colormap.map in
      let ximg = create ~kind: `FASTEST ~visual ~width ~height in

      let f_height = float height in
      for y = 0 to height - 1 do
      	for x = 0 to width - 1 do 
	  let rgb = 
	    try
	      cmap.(bmp#unsafe_get x y) 
	    with
	      _ -> {r=255; g=0; b=0} 
	  in
    	  let red = rgb.r lsl 8 in
    	  let green = rgb.g lsl 8 in
    	  let blue = rgb.b lsl 8 in
    	  Image.put_pixel ximg.ximage ~x ~y 
	    ~pixel:(quick_color_create ~red ~green ~blue)
	done;
	prog (float (y+1) /. f_height)
      done;
      ximg

  | _ -> raise (Failure "not supported")
;;

open GDraw

let mask_of_image win img = (* It is really inefficient *)
  let mono_gc = get_mono_gc win in 
  let width, height = img#width, img#height in
  let draw_mask i =
    prerr_endline "making mask";
    let bmp = Bitmap.create win ~width ~height in
    let ximg = get bmp ~x:0 ~y:0 ~width ~height in
    for x = 0 to width - 1 do
      for y = 0 to height - 1 do
	if i#unsafe_get x y = i#transparent then
  	  Gdk.Image.put_pixel ximg.ximage ~x ~y ~pixel: 0
	else
  	  Gdk.Image.put_pixel ximg.ximage ~x ~y ~pixel: 1
      done;
    done;
    Gdk.Draw.image bmp mono_gc ~image: ximg.ximage 
      ~xsrc:0 ~ysrc:0 ~xdest:0 ~ydest:0 ~width ~height;
    Some bmp
  in

  (* BUG ? of gtk or lablgtk? Using None for mask does not work *)

  begin match OImage.tag img with
  | Index8 i ->
      if i#transparent >= 0 then draw_mask i
      else Some (plain_mask win img#width img#height)
  | Index16 i ->
      let i = OImage.index16 img in
      if i#transparent >= 0 then draw_mask i 
      else Some (plain_mask win img#width img#height)
  | _ -> 
      Some (plain_mask win img#width img#height)
  end;
;;

let pixmap_of_image win progress img =
  let visual = Gdk.Window.get_visual win in
  let ximage = of_image visual progress img in
  let msk = mask_of_image win img in
  let pixmap = new GDraw.pixmap ?mask: msk (pixmap_of win ximage) in
  pixmap
;;

